@@ -11,6 +11,7 @@ import socket
 import time
 import pprint
 import logging
+import shlex
 import os.path
 import requests
 import traceback
@@ -109,15 +110,16 @@ class ChromeExecutionManager():
 
 		if binary is None:
 			binary = "chromium"
+		binary, *args = shlex.split(binary)
+
 		if not os.path.exists(binary):
 			fixed = distutils.spawn.find_executable(binary)
 			if fixed:
 				binary = fixed
 		if not binary or not os.path.exists(binary):
-			raise RuntimeError("Could not find binary '%s'" % binary)
+			self.log.warning("Could not find binary '%s'" % binary)
 
-		argv = [
-				binary,
+		argv = [binary] + args + [
 				'--remote-debugging-port={dbg_port}'.format(dbg_port=dbg_port),
 				'--enable-features=NetworkService',
 			]
@@ -535,6 +537,39 @@ class ChromeExecutionManager():
 		finally:
 			self.soclist[tab_key].settimeout(self.websocket_timeout)
 
+	def __check_console_log(self, message):
+		'''
+		Examine received message to determine if it was the output of a `console.xxx()` call, and
+		optionally output it to stdout.
+		'''
+
+		# I haven't figured out how this is ever called with a None argument.
+		if not message:
+			return
+
+		if 'method' not in message:
+			return
+		if message['method'] != "Runtime.consoleAPICalled":
+			return
+		if 'params' not in message:
+			return
+
+		params = message['params']
+
+		if 'type' not in params:
+			return
+		if params['type'] != 'log':
+			return
+
+		if 'args' not in params:
+			return
+
+		output = " ".join([str(tmp.get("value", "")) for tmp in params['args']])
+		self.log.info("Console output: %s", output)
+
+
+
+
 	def recv_filtered(self, keycheck, tab_key, timeout=30, message=None):
 		'''
 		Receive a filtered message, using the callable `keycheck` to filter received messages
@@ -572,6 +607,7 @@ class ChromeExecutionManager():
 		timeout_at = time.time() + timeout
 		while 1:
 			tmp = self.___recv(tab_key)
+			self.__check_console_log(tmp)
 			if keycheck(tmp):
 				return tmp
 			else:
